//
//  ViewController.m
//  nRF UART
//
//  Created by Ole Morten on 1/11/13.
//  Copyright (c) 2013 Nordic Semiconductor. All rights reserved.
//

#import "ViewController.h"
#import "AboutViewController.h"
#import "DeviceViewController.h"
#import "FileViewController.h"
#import "CommandViewController.h"

#define MAX_PACKET_DLEN   32

typedef enum
{
    IDLE = 0,
    SCANNING,
    CONNECTED,
} ConnectionState;

typedef enum
{
    LOGGING,
    RX,
    TX,
    SEPARATOR
} ConsoleDataType;

typedef enum
{
    DOWNLOAD_IDLE =      0,
    DOWNLOAD_OPEN_FILE,    /*   Open file on BL600 */
    DOWNLOAD_SEND_DATA,
    DOWNLOAD_LAST_DATA,
    DOWNLOAD_CLOSE_FILE,   /*  Close file on BL600 */
} DownloadState_t;

@interface ViewController ()
@property CBCentralManager *cm;
@property ConnectionState state;
@property UARTPeripheral *currentPeripheral;

@property NSTimer     *receiveTimer;
@property NSString    *receiveBuffer;
@property NSLock      *receiveLocker;
@end

@implementation ViewController
{
    DownloadState_t    downloadState;
    NSArray           *lines;
    NSString          *sent_line;
    NSUInteger         sent_bytes;
    NSUInteger         total_bytes;
    NSUInteger         line_count;
    NSUInteger         total_count;
    NSUInteger         console_index;
    // NSUInteger         is_error;
    NSUInteger         is_stopped;
    NSFileHandle      *handleInFile;
    NSFileHandle      *handleMyLog;
}

@synthesize cm = _cm;
@synthesize state = _state;
@synthesize currentPeripheral = _currentPeripheral;

@synthesize receiveTimer = _receiveTimer;
@synthesize receiveBuffer = _receiveBuffer;

@synthesize devicePerpArray = _devicePerpArray;
@synthesize deviceRssiArray = _deviceRssiArray;
@synthesize rcvTimerCount;
@synthesize scanComplete;

- (void)viewDidLoad
{
    [super viewDidLoad];

    // Define my own colors to match Android
    self.myRedColor = [UIColor colorWithRed:250/255.0 green:0/255.0 blue:0/255.0 alpha:1.0];
    self.myBlueColor = [UIColor colorWithRed:16/255.0 green:160/255.0 blue:244/255.0 alpha:1.0];
    self.myGreenColor = [UIColor colorWithRed:0/255.0 green:150/255.0 blue:0/255.0 alpha:1.0];

    //**************** Hide the navigation bar which is behind the image view ****************
    [[self navigationController] setNavigationBarHidden:YES animated:YES];
    [self.myNavigationBar setBackgroundColor:self.myBlueColor];

    // self.commandButton.hidden = YES;

    /*
    if (floor(NSFoundationVersionNumber) > NSFoundationVersionNumber_iOS_6_1)
        [self.navigationController.navigationBar setBarTintColor:navColor];
    else
        [self.navigationController.navigationBar setTintColor:navColor];
    */
    
    //**************** Add a border and corner radius for 'Send' and 'Scan and Connect' buttons ****************
    // self.connectButton.layer.borderWidth = 0.5;
    // self.connectButton.layer.cornerRadius = 0;
    // self.sendButton.layer.borderWidth = 0.5;
    // self.sendButton.layer.cornerRadius = 0;
	
    // Do any additional setup after loading the view, typically from a nib.
    self.cm = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
    
    [self addTextToConsole:@"Touch Connect to make connection" dataType:LOGGING];
    
    self.receiveTimer = nil;
    self.receiveBuffer = nil;
    self.currentPeripheral = nil;
    _receiveLocker = [[NSLock alloc] init];
    
    NSDictionary *infoDictionary = [[NSBundle mainBundle]infoDictionary];
    NSString *build = infoDictionary[(NSString *)kCFBundleVersionKey];
    NSString *version = [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleShortVersionString"];
    NSString *versionBuild = @"  Version ";
    versionBuild = [versionBuild stringByAppendingString:version];
    versionBuild = [versionBuild stringByAppendingString:@"."];
    versionBuild = [versionBuild stringByAppendingString:build];
    [self.labelVersion setTextColor:self.myBlueColor];
    self.labelVersion.text = versionBuild;
    
    // [[self navigationController] setNavigationBarHidden:YES animated:YES];
    self.devicePerpArray = [[NSMutableArray alloc] init];
    self.deviceRssiArray = [[NSMutableArray alloc] init];
    self.consoleTextView.editable = NO;
    
    [self.sendButton setBackgroundColor:[UIColor whiteColor]];
    [self.sendButton setUserInteractionEnabled :NO];
    self.commandButton.hidden = YES;
    [self.stopButton setBackgroundColor:[UIColor whiteColor]];
    [self.stopButton setUserInteractionEnabled :NO];
    self.progressSending.progress = 0.0;
    self.labelProgress.text = @"";
    
    handleMyLog = nil;
    self.fileName.text = @"";
    self.labelStatus.text = @"";
    self.labelService.text = @"";
    self.labelError.text = @"";
}

- (void) viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    self.aboutButton.layer.borderWidth = 1.5;
    [self.aboutButton setTitleColor:[UIColor blueColor] forState:UIControlStateNormal];
    [self.aboutButton setBackgroundColor:[UIColor whiteColor]];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)selectFilePressed:(id)sender {
    FileViewController *fileViewController;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
        fileViewController = [[FileViewController alloc] initWithNibName:@"FileView_iPhone" bundle:nil];
    else if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        fileViewController = [[FileViewController alloc] initWithNibName:@"FileView_iPad" bundle:nil];

    fileViewController.rootView = self;
    [self.navigationController pushViewController:fileViewController animated:YES];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    NSLog(@"prepareForSegue 1:");
    
    if([segue.identifier isEqualToString:@"connectDevice"])
    {
    }
}

- (void) showDevices
{
    DeviceViewController *deviceViewController;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
        deviceViewController = [[DeviceViewController alloc] initWithNibName:@"DeviceView_iPhone" bundle:nil];
    else if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        deviceViewController = [[DeviceViewController alloc] initWithNibName:@"DeviceView_iPad" bundle:nil];
    
    deviceViewController.rootView = self;
    [self.navigationController pushViewController:deviceViewController animated:YES];
}

- (void)disconnectTimout:(NSTimer *) theTimer
{
    self.state = IDLE;
    [self.connectButton setTitle:@"Scan and Connect" forState:UIControlStateNormal];
    self.isConnectButtonBusy = 0;
}

- (IBAction)connectButtonPressed:(id)sender
{
    if(self.isConnectButtonBusy > 0) return;
    
    self.connectButton.enabled = NO;
    self.isConnectButtonBusy = 1;
    // [self.sendTextField resignFirstResponder];
    NSLog(@"connectButtonPressed 1:");
    
    switch (self.state) {
        case IDLE:
            self.state = SCANNING;
            NSLog(@"Started scan ...");
            [self.connectButton setTitle:@"Scanning ..." forState:UIControlStateNormal];
           
            if( [self. devicePerpArray count] > 0 )
            {
                [self.devicePerpArray removeAllObjects];
                [self.deviceRssiArray removeAllObjects];
            }

            [self.cm scanForPeripheralsWithServices:@[UARTPeripheral.uartServiceUUID] options:@{CBCentralManagerScanOptionAllowDuplicatesKey: [NSNumber numberWithBool:NO]}];
            [NSThread sleepForTimeInterval:0.2];
            [self showDevices];
            self.isConnectButtonBusy = 0;
            break;
            
        case SCANNING:
            self.state = IDLE;
            NSLog(@"Stopped scan");
            [self.cm stopScan];
            
            [NSThread sleepForTimeInterval:1];
            [self.connectButton setTitle:@"Scan and Connect" forState:UIControlStateNormal];
            self.isConnectButtonBusy = 0;
            break;
            
        case CONNECTED:
            NSLog(@"Disconnect peripheral %@", self.currentPeripheral.peripheral.name);
            [self.cm cancelPeripheralConnection:self.currentPeripheral.peripheral];

            [self.connectButton setTitle:@"Disconnecting ..." forState:UIControlStateNormal];
            self.receiveTimer = [NSTimer scheduledTimerWithTimeInterval:10
                                                                 target:self
                                                               selector:@selector(disconnectTimout:)
                                                               userInfo:nil
                                                                repeats:NO];
            self.isConnected = 0;
            break;
    }

    self.connectButton.enabled = YES;
}

- (IBAction)stopButtonPressed:(id)sender {
    [self.stopButton setBackgroundColor:[UIColor whiteColor]];
    [self.stopButton setUserInteractionEnabled :NO];
    [self killReceiveTimer];

    if(handleMyLog != nil)
    {
        [handleMyLog closeFile];
        handleMyLog = nil;
    }

    [self.labelStatus setTextColor:[UIColor redColor]];
    self.labelStatus.text = @"Sending Stoped";

    [self.selectButton setBackgroundColor:[UIColor lightGrayColor]];
    [self.selectButton setUserInteractionEnabled :YES];
    [self.sendButton setBackgroundColor:[UIColor lightGrayColor]];
    [self.sendButton setUserInteractionEnabled :YES];
    self.commandButton.hidden = NO;
    is_stopped = 1;
    [UIApplication sharedApplication].idleTimerDisabled = NO;
}

- (IBAction)aboutButtonPressed:(id)sender {
    AboutViewController *aboutViewController;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
        aboutViewController = [[AboutViewController alloc] initWithNibName:@"AboutView_iPhone" bundle:nil];
    else if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        aboutViewController = [[AboutViewController alloc] initWithNibName:@"AboutView_iPad" bundle:nil];
    
    [self.navigationController pushViewController:aboutViewController animated:YES];
}

- (IBAction)commandButtonPressed:(id)sender {
    CommandViewController *commandViewController;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
        commandViewController = [[CommandViewController alloc] initWithNibName:@"CommandView_iPhone" bundle:nil];
    else if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        commandViewController = [[CommandViewController alloc] initWithNibName:@"CommandView_iPad" bundle:nil];

    commandViewController.rootView = self;
    [self.navigationController pushViewController:commandViewController animated:YES];
}

- (void) connectSelectedDevice:(NSInteger)index
{
    if( index >= 0 && index < [self. devicePerpArray count] )
    {
        CBPeripheral * peripheral = self.devicePerpArray[index];
        self.currentPeripheral = [[UARTPeripheral alloc] initWithPeripheral:peripheral delegate:self];
        NSLog(@"Connecting to peripheral %@", peripheral.name);
        [self.cm connectPeripheral:peripheral options:@{CBConnectPeripheralOptionNotifyOnDisconnectionKey: [NSNumber numberWithBool:YES]}];
    }
}

- (void) getNumDevices:(NSInteger *)numDevices
{
    *numDevices = [self. devicePerpArray count];
}

- (void) getDeviceData:(NSInteger)index rtDeviceName:(NSString **)deviceName rtRSSI:(NSInteger *)rssi
{
    *deviceName = nil;
    rssi = nil;
    
    
    if( index > 0 && index < [self. devicePerpArray count] )
    {
        CBPeripheral * peripheral = self.devicePerpArray[index];
        *deviceName = peripheral.name;
        *rssi = self.deviceRssiArray[index];
    }
}


- (IBAction)downloadButtonPressed:( id)sender {
    
    [ UIApplication sharedApplication]. idleTimerDisabled = YES;

    [self.selectButton  setUserInteractionEnabled :NO ];
    [self.sendButton  setUserInteractionEnabled :NO ];
    self.commandButton.hidden = YES;
    self.progressSending. progress = 0.0 ;
    self.labelStatus. text = @"" ;
    self.labelError. text = @"" ;
    self.consoleTextView.text = @"" ;
    console_index = 0;

    /*
    NSDate *curdate = [NSDate date];
    NSDate *timeout = [curdate dateByAddingTimeInterval:10];   // 10 seconds timeout
    BOOL bResult = [_receiveLocker lockBeforeDate:timeout];
    if(bResult == YES)
    {
        _receiveBuffer = nil;
        [_receiveLocker unlock];
    }
     */
    
    if( [self.fileName. text length ] == 0 )
    {
        NSLog (@" sendButtonPressed: No File Name" );
        return;
    }
    
    NSFileManager *fileMgr;
    NSString *docsDir;
    NSArray  *dirPaths;
    fileMgr = [NSFileManager  defaultManager];
    dirPaths = NSSearchPathForDirectoriesInDomains (NSDocumentDirectory , NSUserDomainMask,  YES);
    docsDir = [dirPaths  objectAtIndex:0 ];
    
    NSString *filePath = [docsDir stringByAppendingPathComponent:self.fileName.text];
    NSLog ( @" filePath: %@", filePath);
    
    
    // Open the uwc file
    handleInFile = [NSFileHandle fileHandleForReadingAtPath: filePath];
    if (handleInFile ==  nil )
    {
        NSLog (@" downloadButtonPressed: File Could Not Be Opened" );
        return;
    }
    
    [self.selectButton setBackgroundColor:[UIColor whiteColor]];
    [self.selectButton setUserInteractionEnabled :NO];
    [self.sendButton setBackgroundColor:[UIColor whiteColor]];
    [self.sendButton setUserInteractionEnabled :NO];
    self.commandButton.hidden = YES;
    [self.stopButton setBackgroundColor:[UIColor lightGrayColor]];
    [self.stopButton setUserInteractionEnabled :YES];

    // Open a log file for writing sent_line and responses
    NSString *logPath = [filePath stringByReplacingOccurrencesOfString:@".uwc"
                                                   withString:@".log"];
    // NSLog(@"My Log file name = %@", filePath);
    [fileMgr createFileAtPath:logPath contents:nil attributes:nil ];
    handleMyLog = [NSFileHandle fileHandleForWritingAtPath:logPath];

    [self killReceiveTimer];
    self.receiveTimer = [NSTimer scheduledTimerWithTimeInterval:0.01
                                                         target:self
                                                       selector:@selector(updateReceivingConsole:)
                                                       userInfo:nil
                                                        repeats:YES ];
    // is_error = 0;
    is_stopped = 0;
    sent_bytes = 0;
    [self FileToCommands:filePath];
    // Open the saveAsFile on BL600
    sent_line = [self formOtherPacket:DOWNLOAD_OPEN_FILE];
    [self addTextToConsole: sent_line  dataType :TX ];

    NSDate *curdate = [NSDate date];
    NSDate *timeout = [curdate dateByAddingTimeInterval:10];   // 10 seconds timeout
    BOOL bResult = [_receiveLocker lockBeforeDate:timeout];
    if(bResult == YES)
    {
        _receiveBuffer = sent_line;
        [_receiveLocker unlock];
    }
    [self.currentPeripheral writeString :sent_line];
}

- (void) FileToCommands:(NSString *)filePath
{
    NSData *data = [NSData dataWithContentsOfFile:filePath];
    if(data == nil || data.length == 0) return;
    
    
    NSRange  dataRange;
    NSMutableArray  *myLines = [[NSMutableArray alloc] init];
    NSUInteger len = data.length;
    dataRange.location = 0;
    total_bytes = data.length;
    
    while(len > 0)
    {
        unsigned char srcData[MAX_PACKET_DLEN];
        if(len > MAX_PACKET_DLEN) dataRange.length = MAX_PACKET_DLEN;
        else dataRange.length = len;
        [data getBytes:srcData range:dataRange];
        NSString *line = [self formFWRHpacket:srcData withLength:dataRange.length];
        [myLines addObject:line];
        
        
        dataRange.location += dataRange.length;
        len -= dataRange.length;
    }
    
    lines = [NSArray arrayWithArray:myLines];
    total_count = [lines count];
    NSLog(@"total_count = %d", (int)total_count);
}


- (void) didReadHardwareRevisionString:(NSString *)string
{
    [self addTextToConsole:[NSString stringWithFormat:@"Hardware revision: %@", string] dataType:LOGGING];
}

// Added and modified end on 5/24/2014


- (void) addTextToConsole:( NSString *) string dataType:(ConsoleDataType ) dataType
{
    if(handleMyLog !=  nil)
    {
        // string = [string stringByAppendingString:@"\n"];
        [handleMyLog writeData :[string dataUsingEncoding: NSUTF8StringEncoding]];
    }
    
    static  bool fLastWasLog=true ;
    if(dataType ==  LOGGING)
    {
        NSString *format;
        if(fLastWasLog)
        {
            format =  @"#: %@\n";
        }
        else
        {
            format =  @"\n#: %@\n";
        }
        
        [self.consoleTextView  setText:[ self.consoleTextView .text  stringByAppendingFormat:format, string]];
        fLastWasLog= true;
    }
    else
    {
        [self.consoleTextView  setText:[ self.consoleTextView .text  stringByAppendingFormat: @"%@",string]];
        fLastWasLog= false;
    }
    
    // Truncate the top of textview
    if(self.consoleTextView.text.length > 4096)
    {
        NSUInteger fromIndex = self.consoleTextView.text.length - 4096;
        self.consoleTextView.text = [self.consoleTextView.text substringFromIndex:fromIndex];
    }
    else
        self.consoleTextView.text = [self.consoleTextView.text substringFromIndex:0];
   
    // Scroll down to the last line
    /*
    NSInteger len = [self .consoleTextView .text  length];
    if(len >  10)
    {
        [self.consoleTextView  scrollRangeToVisible: NSMakeRange(len - 1, 1 )];
        self.consoleTextView .scrollEnabled  = NO;
        self.consoleTextView .scrollEnabled  = YES;
    }
     */
}

- (void) centralManagerDidUpdateState:(CBCentralManager *)central
{
    if (central.state == CBCentralManagerStatePoweredOn)
    {
        [self.connectButton setEnabled:YES];
    }
    
}

- (void) centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI
{
    // NSLog(@"Did discover peripheral 1: %@", peripheral.name);
    NSString  *deviceRssi = [NSString stringWithFormat:@"%@", RSSI];
    [self.devicePerpArray addObject:peripheral];
    [self.deviceRssiArray addObject:deviceRssi];
}

- (void) makeBtConnection:(NSInteger)index
{
    [self.cm stopScan];
    [NSThread sleepForTimeInterval:0.5];
    CBPeripheral *peripheral = [self.devicePerpArray objectAtIndex:index];
    self.currentPeripheral = [[UARTPeripheral alloc] initWithPeripheral:peripheral delegate:self];
    
    
    [self.connectButton setTitle:@"Connecting ..." forState:UIControlStateNormal];
    // If not connected in 20 seconds, should stop doing it
    self.receiveTimer = [NSTimer scheduledTimerWithTimeInterval:20
                                                         target:self
                                                       selector:@selector(connectTimout:)
                                                       userInfo:nil
                                                        repeats:NO];
    
    [self.cm connectPeripheral:peripheral options:@{CBConnectPeripheralOptionNotifyOnDisconnectionKey: [NSNumber numberWithBool:YES]}];
}

- (void)connectTimout:(NSTimer *) theTimer
{
    [self.cm cancelPeripheralConnection:self.currentPeripheral.peripheral];
    self.state = IDLE;
    [self.connectButton setTitle:@"Scan and Connect" forState:UIControlStateNormal];
}

- (void) stopBtScan
{
    [self.cm stopScan];
    [NSThread sleepForTimeInterval:0.5];
    self.state = IDLE;
    [self.connectButton setTitle:@"Scan and Connect" forState:UIControlStateNormal];
}

- (void)killReceiveTimer
{
    if(self.receiveTimer)
    {
        [self.receiveTimer invalidate];
        self.receiveTimer = nil;
    }
}

- (void)countScanTime
{
    int numDevices = (int) [self.devicePerpArray count];
    ++self. rcvTimerCount;
    if( ( numDevices > 0 && self.rcvTimerCount > MIN_SCAN_TIME) ||
       self.rcvTimerCount > MAX_SCAN_TIME || numDevices > MAX_SCAN_DEVICES )
    {
        self.scanComplete = 0;
    }
}

// Added and modified for receiving data on 5/24/2014
- (void) updateReceivingConsole:(  NSTimer *) theTimer
{
    static  int idleCount = 0 ;
    ++idleCount;
    console_index = 0;
    
    if(_receiveBuffer !=   nil)
    {
        int need_to_send = 0 ;
        int is_error = 0 ;
        
        int len = (int)_receiveBuffer.length;
        if(len > console_index)
        {
            NSDate *curdate = [NSDate   date];
            NSDate *timeout = [curdate dateByAddingTimeInterval  :10 ];   // 10 seconds timeout
            BOOL bResult = [_receiveLocker lockBeforeDate:timeout];
            if(bResult ==   YES)
            {
                int crFound = 0 ;
                int showString = 1;

                NSString *string = [_receiveBuffer substringWithRange :NSMakeRange (console_index , len - console_index)];
                NSRange range = [string rangeOfString :@"\r" ];
                if( range.location != NSNotFound )
                {
                    ++crFound;
                    int loc1 = (int ) (range.location + range. length);
                    int len1 = len - (int )console_index  - loc1;
                    if(len1 >  0)
                    {
                        NSString *string1 = [string substringWithRange :NSMakeRange (loc1, len1)];
                        range = [string1  rangeOfString:@"\r" ];
                        if( range. location != NSNotFound )
                        {
                            idleCount =  0;
                            ++crFound;
                            console_index = len;
                        }
                        else
                        {
                            if(idleCount < 300) showString = 0;
                        }
                    }
                    else
                    {
                        if(idleCount < 300) showString = 0;
                    }
                    
                }
                else
                {
                    showString = 0;
                }
                
                
                if(idleCount >  300)
                {
                    idleCount =  0;
                    console_index = len;   // drop gabage
                    crFound =  2;   // force to treat response
                }
                
                
                // Check if the received data is complete
                if( crFound >=  2)
                {
                    NSString *response = string;
                    // Check if send_line is echoed
                    NSRange rangeLine = [string rangeOfString :sent_line ];
                    if( rangeLine.location != NSNotFound  )
                    {
                        response = [string stringByReplacingCharactersInRange :rangeLine withString: @""];
                    }
                    
                    // Get the last line of the response
                    NSArray *rspArray = [response componentsSeparatedByString :@"\n" ];
                    NSString *last_response = [rspArray lastObject];
                    
                    // Check the response
                    if( [last_response rangeOfString :@"00\r" ].location  != NSNotFound )
                    {
                        // need_to_send =  1;
                        // Specially for OTA
                        if(downloadState  != DOWNLOAD_CLOSE_FILE &&  is_stopped == 0 )
                            need_to_send =  1;
                        else
                        {
                            if(is_stopped  == 0)
                            {
                                [self .labelStatus  setTextColor:self .myGreenColor];
                                self.labelStatus .text =  @"Sending Finished";
                                [self killReceiveTimer];
                            }
                            is_stopped = 1 ;
                        }
                        // Specially for OTA End
                    }
                    else
                    {
                        NSString *errString = nil;
                        is_error =  1;
                        
                        [self.labelError setTextColor:[UIColor redColor]];
                        NSArray *rspArray = [last_response componentsSeparatedByString:@"\t"];
                        if([rspArray count] > 1)
                        {
                            // Get the error number
                            errString = [rspArray objectAtIndex:1];
                        }
                        
                        if(errString == nil)
                            self.labelError.text = @"UNKNOWN RESPONSE";
                        else
                        {
                            self.labelError.text = [errString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
                        }
                        NSLog(@"labelError:%@", self.labelError.text);
                        
                        [self.labelStatus  setTextColor:[ UIColor redColor ]];
                        self.labelStatus .text  = @"Sending Failed";
                        [self.selectButton  setBackgroundColor:[ UIColor lightGrayColor ]];
                        [self.selectButton  setUserInteractionEnabled : YES];
                        [self.sendButton  setBackgroundColor:[ UIColor lightGrayColor ]];
                        [self.sendButton  setUserInteractionEnabled : YES];
                        self.commandButton.hidden = NO;
                        [self.stopButton  setBackgroundColor:[ UIColor whiteColor ]];
                        [ self.stopButton  setUserInteractionEnabled : NO];
                    }

                    showString = 0;
                }
                
                [_receiveLocker unlock];
                
                if(showString > 0)
                {
                    NSLog(@"Get: %@" , string);
                    [ self addTextToConsole  :string dataType:  RX];
                }
                if(need_to_send >  0) [self sendOnePacket:is_error];
                
                if(is_error >  0 || is_stopped > 0)
                {
                    [self killReceiveTimer];
                    [UIApplication sharedApplication ].idleTimerDisabled  = NO;
                    [self refreshDownloadStates];
                }
            }
        }
    }
    
    if(idleCount >= 1000)
    {
        idleCount = 0;
        [self killReceiveTimer];
        [self.labelError setTextColor:[UIColor redColor]];
        self.labelError.text = @"No Response";
        [self.labelStatus setTextColor:[UIColor redColor]];
        self.labelStatus.text = @"Sending Failed";
        
        [self.selectButton setBackgroundColor:[UIColor lightGrayColor]];
        [self.selectButton setUserInteractionEnabled :YES];
        [self.sendButton setBackgroundColor:[UIColor lightGrayColor]];
        [self.sendButton setUserInteractionEnabled :YES];
        self.commandButton.hidden = NO;
        [self.stopButton setBackgroundColor:[UIColor whiteColor]];
        [self.stopButton setUserInteractionEnabled :NO];
    }
}

- (void) refreshDownloadStates
{
    [ self.selectButton setBackgroundColor:[UIColor lightGrayColor]];
    [ self.selectButton setUserInteractionEnabled:YES ];
    [ self.sendButton setBackgroundColor:[UIColor lightGrayColor]];
    [ self.sendButton setUserInteractionEnabled:YES ];
    self.commandButton.hidden = NO;
    [ self.stopButton setBackgroundColor:[UIColor   whiteColor]];
    [ self.stopButton setUserInteractionEnabled:NO ];

    if(handleInFile != nil)
    {
        [handleInFile closeFile];
        handleInFile = nil;
    }

    if(handleMyLog != nil)
    {
        [handleMyLog closeFile];
        handleMyLog = nil ;
    }

    [ UIApplication sharedApplication]. idleTimerDisabled = NO ;
}

- (void) didReceiveData:(NSString *)string
{
    NSDate *curdate = [NSDate date];
    NSDate *timeout = [curdate dateByAddingTimeInterval:10];   // 10 seconds timeout
    BOOL bResult = [_receiveLocker lockBeforeDate:timeout];
    
    if(bResult == YES)
    {
        if(_receiveBuffer == nil)
            _receiveBuffer = string;
        else
            _receiveBuffer = [_receiveBuffer stringByAppendingString :string];
        
        [_receiveLocker unlock];
    }
}

- (void) centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    NSLog(@"Did connect peripheral %@", peripheral.name);

    [self addTextToConsole:[NSString stringWithFormat:@"Connected to %@", peripheral.name] dataType:LOGGING];
    
    self.state = CONNECTED;
    self.isConnected = 1;
    [self.labelService setTextColor:self.myGreenColor];
    self.labelService.text = @"YES";
    self.labelError.text = @"";
    [self killReceiveTimer];
    
    if(self.fileName.text.length > 0)
    {
        [self.sendButton setBackgroundColor:[UIColor lightGrayColor]];
        [self.sendButton setUserInteractionEnabled :YES];
        self.commandButton.hidden = NO;
        [self.labelStatus setTextColor:[UIColor grayColor]];
        self.labelStatus.text = @"Waiting";
    }
    
    [self.connectButton setTitle:@"Disconnect" forState:UIControlStateNormal];
    [self.sendButton setUserInteractionEnabled:YES];
    self.commandButton.hidden = NO;
    
    if ([self.currentPeripheral.peripheral isEqual:peripheral])
    {
        [self.currentPeripheral didConnect];
    }
}

- (void) centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"Did disconnect peripheral %@", peripheral.name);
    
    [self addTextToConsole:[NSString stringWithFormat:@"Disconnected from %@", peripheral.name] dataType:LOGGING];

    // self.connectButton.enabled = NO;
    [NSThread sleepForTimeInterval:0.2];
    self.state = IDLE;
    self.isConnected = 0;
    self.labelService.text = @"";
    [self.connectButton setTitle:@"Scan and Connect" forState:UIControlStateNormal];
    // self.connectButton.enabled = YES;
    
    if (self.currentPeripheral != nil && [self.currentPeripheral.peripheral isEqual:peripheral])
    {
        [self.currentPeripheral didDisconnect];
        [self killReceiveTimer];
        // Reset buffers
        _receiveBuffer = nil;
        if(self.currentPeripheral != nil) self.currentPeripheral.sendBuffer = nil;
        self.currentPeripheral = nil;
    }

    [self.stopButton setBackgroundColor:[UIColor whiteColor]];
    [self.stopButton setUserInteractionEnabled :NO];
    [self.sendButton setBackgroundColor:[UIColor whiteColor]];
    [self.sendButton setUserInteractionEnabled:NO];
    self.commandButton.hidden = YES;
    
    self.labelStatus.text = @"";
    self.labelService.text = @"";
    if( sent_bytes < total_bytes )
    {
        [self.labelError setTextColor:self.myRedColor];
        self.labelError.text = @"DEVICE DISCONNECTED";
    }
    else
        self.labelError.text = @"";

    self.isConnectButtonBusy = 0;
}



- (NSString *) formFWRHpacket:(unsigned char *)srcData withLength:(unsigned long)dlen
{
    static const char HexChars[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
        'A', 'B', 'C', 'D', 'E', 'F' };
    char dstData[128];
    strcpy(dstData, "AT+FWRH \"");
    uint didx = (uint)strlen(dstData);

    for (unsigned long i = 0; i < dlen; ++i) {
        dstData[didx++] = HexChars[(srcData[i] >> 4)];
        dstData[didx++] = HexChars[(srcData[i] & 0x0F)];
    }
    strcpy(&dstData[didx], "\"\r");
    
    return [NSString stringWithCString:dstData encoding:NSASCIIStringEncoding];
}


- (NSString *) formOtherPacket:(DownloadState_t)state
{
    NSString *string = nil;
    downloadState = state;
    
    if(state == DOWNLOAD_OPEN_FILE)
    {
        NSString *string1 = @"************\n";
        [self addTextToConsole :string1 dataType: SEPARATOR];
        string = @"AT+FOW \"";
        string = [string stringByAppendingString :self.labelSaveAs.text];
        string = [string stringByAppendingString :@"\"\r" ];
    }
    else if(state == DOWNLOAD_CLOSE_FILE)
    {
        string = @"AT+FCL\r";
    }
    
    return string;
}


- (void) sendOnePacket:(int)is_error
{
    NSString *string = @"************\n";
    [self addTextToConsole :string dataType: SEPARATOR];
    // if (handleMyLog == nil) return;
    
    // Display the progress
    self .progressSending .progress = (float)sent_bytes / (float)total_bytes ;
    // int progrssInPercent = (int)(sent_bytes * 100 / total_bytes);
    self.labelProgress.text = [NSString stringWithFormat :@"%ld/%ld" , (long)sent_bytes, (long)total_bytes];

    sent_line = nil;
    if(is_error == 0)
    {
        switch(downloadState)
        {
            case DOWNLOAD_OPEN_FILE:
                downloadState = DOWNLOAD_SEND_DATA;
                line_count = 0;
            case DOWNLOAD_SEND_DATA:
                if(line_count < total_count)
                {
                    sent_line = [lines objectAtIndex:line_count];
                    ++line_count;
                    sent_bytes += MAX_PACKET_DLEN;
                    if(sent_bytes > total_bytes) sent_bytes = total_bytes;
                }
                else
                {
                    sent_line = [self formOtherPacket:DOWNLOAD_CLOSE_FILE];
                    downloadState = DOWNLOAD_CLOSE_FILE;
                    NSLog(@"sendOnePacket: Close File");
                }
                break;

            case DOWNLOAD_LAST_DATA:
                sent_line = [self formOtherPacket:DOWNLOAD_CLOSE_FILE];
                downloadState = DOWNLOAD_CLOSE_FILE;
                break;
            case DOWNLOAD_CLOSE_FILE:
                break;
            default:
                NSLog(@"sendOnePacket: dpwnloadState = %d", downloadState);
                break;
        }
    }
    
    if(sent_line != nil)
    {
        if(self.consoleTextView.text.length > 512)
        {
            NSUInteger fromIndex = self.consoleTextView.text.length - 512;
            self.consoleTextView.text = [self.consoleTextView.text substringFromIndex:fromIndex];
        }
        [ self addTextToConsole: sent_line  dataType :TX ];

        NSDate *curdate = [NSDate date];
        NSDate *timeout = [curdate dateByAddingTimeInterval:10];   // 10 seconds timeout
        BOOL bResult = [_receiveLocker lockBeforeDate:timeout];
        if(bResult == YES)
        {
            _receiveBuffer = sent_line;
            [_receiveLocker unlock];
        }
        _receiveBuffer = sent_line;
        [ self.currentPeripheral writeString:sent_line ];
        
        [ self.labelStatus setTextColor:[UIColor blueColor]];
        self .labelStatus .text = @"Sending!!" ;
    }
    else
        [self refreshDownloadStates];
}

- (void) sendOneCommand:(NSString *)cmdString
{
    sent_line = cmdString;
    _receiveBuffer = sent_line;
    [ self.currentPeripheral writeString:sent_line ];
}

// Return rval
// rval = 0, response not complete
//        1 response complete
- (int) PollResponse
{
    int len = (int)_receiveBuffer.length;
    int crFound = 0 ;
    
    if(len > 0)
    {
        NSRange range = [_receiveBuffer rangeOfString :@"\r" ];
        
        if( range.location != NSNotFound )
        {
            ++crFound;
            int loc1 = (int ) (range.location + range.length);
            int len1 = len - loc1;
            if(len1 > 0)
            {
                NSString *string1 = [_receiveBuffer substringWithRange: NSMakeRange(loc1, len1)];
                range = [string1 rangeOfString:@"\r" ];
                if( range. location != NSNotFound ) ++crFound;
            }
        }
    }
    
    if(crFound >= 2) return 1;
    else return 0;
}

// Check response for sent_line
- (NSString *) CheckResponse:(int *)errCode
{
    NSString *rstring = nil;
    int rval = [self PollResponse];
    *errCode = 0;
    
    if(rval != 0)   // Response Complete
    {
        // NSString *errString = nil;
        NSString *response = _receiveBuffer;
        rstring = [_receiveBuffer substringWithRange :NSMakeRange(0 , _receiveBuffer.length)];
        
        // Check if send_line is echoed
        NSRange rangeLine = [response rangeOfString: sent_line];
        if( rangeLine.location != NSNotFound )
        {
            // Remove the sent_line echo
            response = [response stringByReplacingCharactersInRange :rangeLine withString: @""];
            // Get the last line of the response
            NSArray *rspArray = [response componentsSeparatedByString :@"\n" ];
            NSString *last_response = [rspArray lastObject];
            
            // Check the response
            if( [last_response rangeOfString :@"00\r" ].location == NSNotFound )
            {
                NSArray *rspArray1 = [last_response componentsSeparatedByString:@"\t"];
                
                if([rspArray1 count] > 1)
                {
                    *errCode = 1;
                    // Get the error number
                    // errString = [rspArray1 objectAtIndex:1];
                }
                else
                {
                    *errCode = 2;
                    // errString = @"UNKNOWN RESPONSE";
                }
            }
        }
        else
        {
            *errCode = 3;   // No echo for sent_line
            // errString = @"NO SENDING ECHO";
        }
        
        // if(errString != nil)
        //     rstring = [rstring stringByAppendingString: errString];
    }
    
    return rstring;
}

@end
